@@ -8,30 +8,35 @@ import os
 import glob
 from math import pow, floor
 from tqdm import tqdm
-from PyQt5.QtGui import QIcon
-from PyQt5.QtWidgets import QApplication, QStyleFactory, QFileDialog
-from PyQt5 import QtCore
 import sys
 import difflib
 from scipy.stats import binned_statistic
 import warnings
+from typing import Iterable
 
-from . import myutils, prompts, apps, qrc_resources, widgets, html_utils, printl
+from . import GUI_INSTALLED
+if GUI_INSTALLED:
+    from qtpy.QtWidgets import QFileDialog
+    from . import widgets
+    from . import _run
+
+from . import load, cca_df_colnames
+from . import myutils, prompts, html_utils, printl
+
+default_summable_columns = (
+    'cell_area_um2', 
+    'cell_vol_fl', 
+    'cell_vol_vox',
+    'cell_area_pxl', 
+    'num_spots', 
+    'ref_ch_vol_um3', 
+    'ref_ch_num_fragments', 
+    'ref_ch_vol_vox'
+)
 
 def configuration_dialog():
-    if os.name == 'nt':
-        try:
-            # Set taskbar icon in windows
-            import ctypes
-            myappid = 'schmollerlab.cellacdc.pyqt.v1' # arbitrary string
-            ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
-        except Exception as e:
-            pass
-    # app = QtCore.QCoreApplication.instance()
-    # if app is None:
-    app = QApplication(sys.argv)
-    app.setStyle(QStyleFactory.create('Fusion'))
-    app.setWindowIcon(QIcon(":icon.ico"))
+    app, _ = _run._setup_app(splashscreen=False)
+    
     continue_selection = True
     data_dirs = []
     positions = []
@@ -92,36 +97,75 @@ def configuration_dialog():
         print('******************************')
         print("No positions selected!")
         print('******************************')
-        raise IndexError
-    return data_dirs, positions
+        raise IndexError("No positions selected!")
+    return data_dirs, positions, app
 
 def find_available_channels(filenames, first_pos_dir):
     ch_name_selector = prompts.select_channel_name()
     ch_names, warn = ch_name_selector.get_available_channels(
         filenames, first_pos_dir
     )
-    return ch_names, warn
+    return ch_names, ch_name_selector.basename
 
+def get_segm_endname(images_path, basename):
+    segm_files = load.get_segm_files(images_path)
+    segm_endnames = load.get_endnames(
+        basename, segm_files
+    )
+    if not segm_endnames:
+        msg = widgets.myMessageBox()
+        txt = html_utils.paragraph(f"""
+            The following position does not contain valid segmentation files.<br><br>
+            <code>{images_path}</code><br>
+        """)
+        msg.critical(None, 'Segmentation file(s) not found', txt)
+        raise FileNotFoundError(f'Segmentation files not found in "{images_path}"')
+
+    if len(segm_endnames) == 1:
+        return segm_endnames[0]
+    
+    selectSegmWin = widgets.QDialogListbox(
+        'Select segmentation file',
+        'Select segmentation file to use as ROI:\n',
+        segm_endnames, multiSelection=False, parent=None
+    )
+    selectSegmWin.exec_()
+    if selectSegmWin.cancel:
+        raise FileNotFoundError(f'Segmentation file selection aborted by the user.')
+    
+    return selectSegmWin.selectedItemsText[0]
+    
+    
 def calculate_downstream_data(
-    file_names,
-    image_folders,
-    positions,
-    channels,
-    force_recalculation=False
-):
+        file_names,
+        image_folders,
+        positions,
+        channels,
+        segm_endname,
+        force_recalculation=False, 
+        calculate_fluo_metrics=True, 
+        save_features_to_acdc_df=False,
+    ):
     no_of_channels = len(channels)
     overall_df = pd.DataFrame()
     for file_idx, file in enumerate(file_names):
         for pos_idx, pos_dir in enumerate(image_folders[file_idx]):
             channel_data = ('placeholder')*no_of_channels
             print(f'Load files for {file}, {positions[file_idx][pos_idx]}...')
+            acdc_df_path = None
             try:
-                *channel_data, seg_mask, cc_data, metadata, cc_props = _load_files(pos_dir, channels)
+                *channel_data, seg_mask, cc_data, metadata, cc_props, acdc_df_path = (
+                    _load_files(
+                        pos_dir, channels, segm_endname, 
+                        load_channels_data=calculate_fluo_metrics
+                    )
+                )
             except TypeError:
                 print(f'File {file}, position {positions[file_idx][pos_idx]} skipped due to missing segmentation mask/CC annotations.')
                 continue
             print(f'Number of cells in position: {len(cc_data.Cell_ID.unique())}')
             print(f'Number of annotated frames in position: {cc_data.frame_i.max()+1}')
+
             cc_data = _rename_columns(cc_data)
             is_timelapse_data, is_zstack_data = False, False
             if int(metadata.loc['SizeT'])>1:
@@ -130,7 +174,7 @@ def calculate_downstream_data(
                 is_zstack_data=True
             if cc_props is not None and not force_recalculation:
                 print('Cell Cycle property data already existing, loaded from disk...')
-                overall_df = pd.concat([overall_df,cc_props], ignore_index=True).reset_index(drop=True)
+                overall_df = pd.concat([overall_df, cc_props], ignore_index=True).reset_index(drop=True)
             else:
                 print(f'Calculate regionprops on each frame based on Segmentation...')
                 rp_df = _calculate_rp_df(seg_mask, is_timelapse_data, is_zstack_data, metadata, max_frame=cc_data.frame_i.max()+1)
@@ -178,6 +222,12 @@ def calculate_downstream_data(
                 temp_df.to_csv(save_path, index=False)
                 overall_df = pd.concat([overall_df, temp_df], ignore_index=True).reset_index(drop=True)
 
+            # if save_features_to_acdc_df:
+            #     acdc_df = cc_data.set_index(['frame_i', 'Cell_ID'])
+            #     missing_cols = overall_df.columns.difference(acdc_df.columns)
+            #     acdc_df_path
+            #     import pdb; pdb.set_trace()
+
     print('Done!')
     return overall_df, is_timelapse_data, is_zstack_data
 
@@ -338,23 +388,41 @@ def _auto_rescale_intensity(img, perc=0.01, clip_min=False):
     scaled_img = (img-vmin)/(vmax-vmin)
     return scaled_img
 
+def load_acdc_output_only(
+        file_names,
+        image_folders,
+        positions,
+        segm_endnames
+    ):
+    """
+    Function to load only the acdc output.
+    Use when fluorescent file is too big to load into RAM.
+    #TODO: move to cca_functions
+    """
+    
+    overall_df = pd.DataFrame()
+    for file_idx, file in enumerate(file_names):
+        acdc_output_endname = segm_endnames[file_idx].replace('segm', 'acdc_output')
+        for pos_idx, pos_dir in enumerate(image_folders[file_idx]):
+            try:
+                cc_stage_path = glob.glob(
+                    os.path.join(f'{pos_dir}', f'*{acdc_output_endname}.csv')
+                )[0]
+            except IndexError:
+                cc_stage_path = glob.glob(os.path.join(f'{pos_dir}', '*cc_stage.csv'))[0]
+            temp_df = pd.read_csv(cc_stage_path)
+            temp_df['max_frame_pos'] = temp_df.frame_i.max()
+            temp_df['file'] = file
+            temp_df['selection_subset'] = file_idx
+            temp_df['position'] = positions[file_idx][pos_idx]
+            temp_df['directory'] = pos_dir
+            overall_df = pd.concat([overall_df, temp_df])
+    return overall_df
 
-def _load_files(file_dir, channels):
-    """
-    Function to load files of all given channels and the corresponding segmentation masks.
-    Check first if aligned files are available and use them if so.
-    """
-    no_of_aligned_files = len(glob.glob(os.path.join(f'{file_dir}', '*aligned.npz')))
-    seg_mask_available = len(glob.glob(os.path.join(f'{file_dir}', '*_segm.npz'))) > 0
-    acdc_output_available = (
-        len(glob.glob(os.path.join(f'{file_dir}', '*acdc_output.csv')))
-        + len(glob.glob(os.path.join(f'{file_dir}', '*cc_stage*'))) > 0
-    )
-    if not (seg_mask_available and acdc_output_available):
-        return None
+def _load_channels_data(file_dir, channel_names, no_of_aligned_files):
     channel_files = []
     if no_of_aligned_files > 0:
-        for channel in channels:
+        for channel in channel_names:
             try:
                 ch_aligned_path = glob.glob(os.path.join(f'{file_dir}', f'*{channel}_aligned.npz'))[0]
                 channel_files.append(np.load(ch_aligned_path)['arr_0'])
@@ -367,18 +435,46 @@ def _load_files(file_dir, channels):
                     print(f'Resulting data will not contain fluorescent data for this channel')
                     channel_files.append(None)
     else:
-        for channel in channels:
+        for channel in channel_names:
             try:
-                ch_not_aligned_path = glob.glob(os.path.join(f'{file_dir}', f'*{channel}*'))[0]
+                ch_not_aligned_path = (
+                    glob.glob(os.path.join(f'{file_dir}', f'*{channel}.tif'))[0]
+                )
                 channel_files.append(imread(ch_not_aligned_path))
             except IndexError:
                 print(f'Could not find any file for channel {channel}')
                 print(f'Resulting data will not contain fluorescent data for this channel')
                 channel_files.append(None)
+    return channel_files
+
+def _load_files(file_dir, channels, segm_endname, load_channels_data=True):
+    """
+    Function to load files of all given channels and the corresponding segmentation masks.
+    Check first if aligned files are available and use them if so.
+    """
+    acdc_output_endname = segm_endname.replace('segm', 'acdc_output')
+    no_of_aligned_files = len(
+        glob.glob(os.path.join(f'{file_dir}', '*aligned.npz'))
+    )
+    seg_mask_available = len(
+        glob.glob(os.path.join(f'{file_dir}', f'*_{segm_endname}.npz'))
+    ) > 0
+    acdc_output_available = (
+        len(glob.glob(os.path.join(f'{file_dir}', f'*{acdc_output_endname}.csv')))
+        + len(glob.glob(os.path.join(f'{file_dir}', '*cc_stage*'))) > 0
+    )
+    if not (seg_mask_available and acdc_output_available):
+        return None
+    channel_files = []
+    if load_channels_data:
+        channels_data = _load_channels_data(file_dir, channels, no_of_aligned_files)
+        channel_files.extend(channels_data)
 
     # append segmentation file
     try:
-        segm_file_path = glob.glob(os.path.join(f'{file_dir}', '*_segm.npz'))[0]
+        segm_file_path = glob.glob(
+            os.path.join(f'{file_dir}', f'*_{segm_endname}.npz')
+        )[0]
         channel_files.append(np.load(segm_file_path)['arr_0'])
     except IndexError:
         segm_file_path = glob.glob(os.path.join(f'{file_dir}', '*_segm.npy'))[0]
@@ -386,7 +482,9 @@ def _load_files(file_dir, channels):
         channel_files.append(np.load(segm_file_path))
     # append cc-data
     try:
-        cc_stage_path = glob.glob(os.path.join(f'{file_dir}', '*acdc_output.csv'))[0]
+        cc_stage_path = glob.glob(
+            os.path.join(f'{file_dir}', f'*{acdc_output_endname}.csv')
+        )[0]
     except IndexError:
         cc_stage_path = glob.glob(os.path.join(f'{file_dir}', '*cc_stage.csv'))[0]
     # assume cell cycle output of ACDC to be .csv
@@ -407,10 +505,7 @@ def _load_files(file_dir, channels):
         channel_files.append(pd.read_csv(cc_props_path))
     else:
         channel_files.append(None)
-    return tuple(channel_files)
-
-
-
+    return (*channel_files, cc_stage_path)
 
 def _calculate_rp_df(seg_mask, is_timelapse_data, is_zstack_data, metadata, max_frame=1, label_input=False):
     """
@@ -538,15 +633,32 @@ def _calculate_flu_signal(seg_mask, channel_data, channels, cc_data, is_timelaps
     channel_data is a list-like of TYX arrays, one for each channel.
     channels are the name of the channels in the tuple.
     cc_data the output of acdc.
-    """
+    """        
     max_frame = cc_data.frame_i.max()
     df = pd.DataFrame(columns=['frame_i', 'Cell_ID'])
     bg_medians = []
+    
+    if seg_mask.ndim == 4:
+        raise TypeError(
+            '4D segmentation masks not supported. '
+            'Feel free to request the new feature on our GitHub page '
+            'https://github.com/SchmollerLab/Cell_ACDC/issues'
+        )
+    
+    for i, ch_img in enumerate(channel_data):
+        if ch_img.ndim == 3:
+            continue
+        
+        # Use sum projections for 4D data
+        channel_data[i] = ch_img.sum(axis=1)
+    
     for ch_idx, ch_array in enumerate(channel_data):
         if ch_array is None:
             bg_medians.append(None)
         else:
-            bg_index = np.logical_and(seg_mask[:max_frame+1]==0, ch_array[:max_frame+1]!=0)
+            bg_index = np.logical_and(
+                seg_mask[:max_frame+1]==0, ch_array[:max_frame+1]!=0
+            )
             ch_medians = [np.median(ch_array[t][bg_index[t]]) for t in range(max_frame+1)]
             bg_medians.append(ch_medians)
     if is_timelapse_data:
@@ -631,3 +743,244 @@ def calculate_effect_size_glass(data, group1, group2, cat_column='size_category'
     glass_s = np.std(data_gr2[val_column])
     effect_size = (np.mean(data_gr1[val_column])- np.mean(data_gr2[val_column])) / glass_s
     return effect_size
+
+def _add_end_of_frame_i_column(acdc_df):
+    cca_df_idx = acdc_df.cell_cycle_stage.dropna().index
+    cca_df = acdc_df.loc[cca_df_idx][cca_df_colnames]
+    acdc_df['end_of_cell_cycle_frame_i'] = np.nan
+    
+    will_divice_cca_df_S = cca_df[
+        (cca_df.cell_cycle_stage == 'S') & (cca_df.will_divide > 0)
+    ].reset_index()
+    
+    cca_df['end_of_cell_cycle_frame_i'] = -1
+    grouped_ID_gen_num = will_divice_cca_df_S.groupby(
+        ['Cell_ID', 'generation_num']
+    )
+                
+    end_cc_frame_i_per_cycle = grouped_ID_gen_num.agg(
+        end_of_cell_cycle_frame_i=('frame_i', 'max')
+    )
+    
+    cca_df_with_gen_num_idx = (
+        cca_df.reset_index()
+        .set_index(['Cell_ID', 'generation_num'])
+        .sort_index()
+    )
+    
+    for row in end_cc_frame_i_per_cycle.itertuples():
+        ID, gen_num = row.Index
+        end_cc_frame_i = row.end_of_cell_cycle_frame_i
+        idx = (ID, gen_num)
+        cca_df_with_gen_num_idx.loc[idx, 'end_of_cell_cycle_frame_i'] = (
+            end_cc_frame_i
+        )
+    
+    cca_df = (
+        cca_df_with_gen_num_idx.reset_index()
+        .set_index(['frame_i', 'Cell_ID'])
+        .sort_index()
+    )
+    
+    acdc_df.loc[cca_df_idx, 'end_of_cell_cycle_frame_i'] = (
+        cca_df['end_of_cell_cycle_frame_i']
+    )
+    return acdc_df
+
+def _extend_will_divide_to_G1(acdc_df):
+    acdc_df = acdc_df.drop(columns=['level_0', 'index'], errors='ignore')
+    acdc_df = acdc_df.reset_index()
+    acdc_df_will_divide_true = acdc_df[acdc_df['will_divide'] > 0]
+    grouped = acdc_df_will_divide_true.groupby(['Cell_ID', 'generation_num'])
+    for (ID, gen_num) in grouped.groups.keys():
+        mask = (
+            (acdc_df['Cell_ID'] == ID)
+            & (acdc_df['generation_num'] == gen_num)
+        )
+        acdc_df.loc[mask, 'will_divide'] = 1.0
+    acdc_df = (
+        acdc_df.reset_index()
+        .set_index(['frame_i', 'Cell_ID'])
+        .sort_index()
+    )
+    return acdc_df
+    
+def add_derived_cell_cycle_columns(acdc_df: pd.DataFrame):
+    if 'cell_cycle_stage' not in acdc_df.columns:
+        return acdc_df
+    
+    acdc_df = _extend_will_divide_to_G1(acdc_df)
+    acdc_df = _add_end_of_frame_i_column(acdc_df)
+    
+    return acdc_df
+    
+def add_generation_num_of_relative_ID(
+        acdc_df, prefix_index: Iterable[str]=None, reset_index=True
+    ):
+    relID_index_col = ['frame_i', 'relative_ID', 'Cell_ID']
+    ID_index_col = ['frame_i', 'Cell_ID', 'relative_ID']
+    
+    if prefix_index is not None:
+        relID_index_col = [*prefix_index, *relID_index_col]
+        ID_index_col = [*prefix_index, *ID_index_col]
+    
+    if reset_index:
+        acdc_df = acdc_df.reset_index()
+    
+    acdc_df_by_rel_ID = acdc_df.set_index(relID_index_col)
+    acdc_df_by_rel_ID.index
+    relative_ID_idx = acdc_df_by_rel_ID.index
+
+    acdc_df_by_frame_i = acdc_df.set_index(ID_index_col)
+    relative_ID_idx = relative_ID_idx.intersection(acdc_df_by_frame_i.index)
+    acdc_df_by_frame_i['generation_num_relID'] = -1
+
+    acdc_df_by_frame_i.loc[relative_ID_idx, 'generation_num_relID'] = (
+        acdc_df_by_rel_ID.loc[relative_ID_idx, 'generation_num']
+    )
+
+    # Fix where generation_num_relID is still -1
+    to_fix_mask = acdc_df_by_frame_i.generation_num_relID == -1
+    acdc_df_to_fix = (
+        acdc_df_by_frame_i[to_fix_mask]
+        .reset_index()
+        .set_index([*prefix_index, 'frame_i', 'relative_ID'])
+    )
+
+    acdc_df_by_cellID = (
+        acdc_df_by_rel_ID.reset_index()
+        .set_index([*prefix_index, 'frame_i', 'Cell_ID'])
+    )
+    # Intersection takes care of disappearing relative_IDs
+    fixing_idx = acdc_df_to_fix.index.intersection(acdc_df_by_cellID.index)
+
+    acdc_df_to_fix.loc[fixing_idx, 'generation_num_relID'] = (
+        acdc_df_by_cellID.loc[fixing_idx, 'generation_num'].values
+    )
+    index_to_fix = acdc_df_by_frame_i[to_fix_mask].index
+    acdc_df_by_frame_i.loc[index_to_fix, 'generation_num_relID'] = (
+        acdc_df_to_fix['generation_num_relID'].values
+    )
+    
+    acdc_df_with_col = acdc_df_by_frame_i.reset_index()
+    return acdc_df_with_col
+    
+def get_IDs_gen_num_will_divide_wrong(global_cca_df):
+    """Get a list of (ID, gen_num) of cells whose `will_divide`>0 but the 
+    next generation does not exist (i.e., `will_divide` is wrong)
+
+    Parameters
+    ----------
+    global_cca_df : pd.DataFrame
+        DataFrame with cc annotations for every frame and Cell_ID
+
+    Returns
+    -------
+    list of tuples
+        List of (ID, gen_num) of cells whose `will_divide`>0 but the 
+        next generation does not exist (i.e., `will_divide` is wrong)
+    
+    Notes
+    -----
+    To get the (ID, gen_num) where `will_divide` is wrong we first get an 
+    index of (ID, gen_num) where `will_divide`>0. 
+     
+    Then we get the same index but with (ID, gen_num+1) which is the next 
+    generation. 
+    
+    Finally we check if (ID, gen_num+1) actually exists in the annotations. 
+    If not, those are wrongly annotated with `will_divide`>0. To check for 
+    the existence we get the difference between the next gen index and the 
+    whole DataFrame (i.e., get the (ID, gen_num+1) that do not exist in 
+    annotations).
+    """    
+    global_cca_will_divide = (
+        global_cca_df[(global_cca_df['will_divide'] > 0)]
+    ).reset_index()
+    
+    ID_gen_num_index = (
+        global_cca_df.reset_index()
+        .set_index(['Cell_ID', 'generation_num'])
+        .index
+    )
+    
+    # Next generation index
+    next_gen_will_divide_cca_df = (
+        global_cca_will_divide[['Cell_ID', 'generation_num']].copy()
+    )
+    next_gen_will_divide_cca_df['generation_num'] += 1
+    next_gen_will_divide_index = (
+        next_gen_will_divide_cca_df.reset_index()
+        .set_index(['Cell_ID', 'generation_num'])
+        .index
+    )
+    
+    # (ID, gen_num) list of cells with will_divide>0 but whose next 
+    # generation number actually does not exist
+    IDs_will_divide_next_gen_does_not_exist = (
+        next_gen_will_divide_index.difference(ID_gen_num_index)
+        .to_frame().to_numpy() # .to_list()
+    )
+    IDs_will_divide_next_gen_does_not_exist[:, -1] -= 1
+    
+    IDs_will_divide_wrong = list(zip(
+        IDs_will_divide_next_gen_does_not_exist[:,0], 
+        IDs_will_divide_next_gen_does_not_exist[:, 1]
+    ))
+    return IDs_will_divide_wrong
+    
+def generate_mother_bud_total_df(
+        df, 
+        column_operation_mapper: dict[str, str], 
+        do_copy_all_nonselected_columns=True,
+        grouping_columns=None,
+        entity_colname='entity'
+    ):
+    if grouping_columns is None:
+        grouping_columns = []
+    
+    df_G1 = df[df['cell_cycle_stage'] == 'G1']
+    df_S = df[(df['cell_cycle_stage'] == 'S')]
+    df_S_bud = df_S[df_S['relationship'] == 'bud']
+    df_S_moth = df_S[df_S['relationship'] == 'mother']
+    
+    df_S_bud_relID = df_S_bud.reset_index().set_index(
+        [*grouping_columns, 'frame_i', 'relative_ID']
+    )
+    df_S_bud_relID.index.names = [*grouping_columns, 'frame_i', 'Cell_ID']
+    df_S_moth = df_S_moth.reset_index().set_index(
+        [*grouping_columns, 'frame_i', 'Cell_ID']
+    )
+    
+    columns_to_add = [
+        col for col, operation in column_operation_mapper.items() 
+        if 'sum' in operation.lower()
+    ]
+    
+    if do_copy_all_nonselected_columns:
+        df_S_tot = df_S_moth.copy()
+    else:
+        columns_to_keep = list(column_operation_mapper.keys())
+        df_S_tot = df_S_moth[columns_to_keep].copy()
+        df_G1 = df_G1[columns_to_keep].copy()
+        df_S_bud = df_S_bud[columns_to_keep].copy()
+        df_S_moth = df_S_moth[columns_to_keep].copy()
+    
+    df_S_tot[columns_to_add] = (
+        df_S_tot[columns_to_add] + df_S_bud_relID[columns_to_add]
+    )
+    
+    df_S_tot = df_S_tot.drop(columns='level_0', errors='ignore').reset_index()
+    df_S_moth = df_S_moth.drop(columns='level_0', errors='ignore').reset_index()
+    df_S_bud = df_S_bud.reset_index()
+    
+    final_df = pd.concat(
+        [df_G1, df_S_moth, df_S_bud, df_S_tot], 
+        keys=['G1', 'Mother', 'Bud', 'Total'],
+        names=[entity_colname],
+        ignore_index=True
+    )
+    
+    return final_df
+    
+    
\ No newline at end of file
