@@ -5,13 +5,17 @@ import skimage.segmentation
 import skimage.measure
 
 from collections.abc import Callable, Sequence
-from pyqtgraph.colormap import ColorMap
-
 import numpy as np
-import matplotlib
-import colorsys
-import matplotlib.pyplot as plt
-from matplotlib.colors import LinearSegmentedColormap
+
+from . import GUI_INSTALLED, printl
+
+if GUI_INSTALLED:
+    from pyqtgraph.colormap import ColorMap
+    import matplotlib
+    import colorsys
+    import matplotlib.pyplot as plt
+    from matplotlib.colors import LinearSegmentedColormap
+    import seaborn as sns
 
 try:
     import networkx as nx
@@ -90,6 +94,7 @@ def lighten_color(color, amount=0.3, hex=True):
         c = matplotlib.colors.cnames[color]
     except:
         c = color
+    
     c = colorsys.rgb_to_hls(*matplotlib.colors.to_rgb(c))
     lightened_c = colorsys.hls_to_rgb(c[0], 1 - amount * (1 - c[1]), c[2])
     if hex:
@@ -134,9 +139,14 @@ def get_lut_from_colors(colors, name='mycmap', N=256, to_uint8=False):
         lut = (lut*255).astype(np.uint8)
     return lut
 
-def invertRGB(self, rgb_img):
-    if self.imgCmapName != 'grey':
-        return
+def plt_colormap_to_pg_lut(name: str, ncolors=256):
+    cmap = plt.get_cmap(name)
+    colors = [cmap(i) for i in np.linspace(0, 1, ncolors)]
+    lut_float = np.array(colors)
+    lut = np.round(lut_float * 255).astype(np.uint8)
+    return lut
+
+def invertRGB(rgb_img, max_val=1.0):
     # see https://forum.image.sc/t/invert-rgb-image-without-changing-colors/33571
     R = rgb_img[:, :, 0]
     G = rgb_img[:, :, 1]
@@ -144,17 +154,30 @@ def invertRGB(self, rgb_img):
     GB_mean = np.mean([G, B], axis=0)
     RB_mean = np.mean([R, B], axis=0)
     RG_mean = np.mean([R, G], axis=0)
-    rgb_img[:, :, 0] = 1-GB_mean
-    rgb_img[:, :, 1] = 1-RB_mean
-    rgb_img[:, :, 2] = 1-RG_mean
+    rgb_img[:, :, 0] = max_val-GB_mean
+    rgb_img[:, :, 1] = max_val-RB_mean
+    rgb_img[:, :, 2] = max_val-RG_mean
     return rgb_img
 
+def rescale_RGB(rgb_img, saturation_val=1.0):
+    rescaled_rgb = rgb_img-rgb_img.min()
+    max_val = rescaled_rgb.max()
+    brightness = saturation_val/max_val
+    return rescaled_rgb*brightness
+    
+
 def get_greedy_lut(lab, lut, ids=None):    
     expanded = skimage.segmentation.expand_labels(lab, distance=7)
     adj_M = np.zeros([expanded.max() + 1]*2, dtype=bool)
     if ids is None:
         ids = [obj.label for obj in skimage.measure.regionprops(lab)]
     
+    if len(ids) == 1:
+        greedy_lut = np.copy(lut)
+        greedy_lut[:] = greedy_lut[-1]
+        greedy_lut[0] = (0, 0, 0, 0)
+        return greedy_lut
+    
     # Taken from https://stackoverflow.com/questions/26486898/matrix-of-labels-to-adjacency-matrix
     adj_M[expanded[:, :-1], expanded[:, 1:]] = 1
     adj_M[expanded[:, 1:], expanded[:, :-1]] = 1
@@ -165,7 +188,7 @@ def get_greedy_lut(lab, lut, ids=None):
     adj_M[expanded[:-1, :], expanded[1:, :]] = 1
     # adj_M = adj_M[1:, 1:]
 
-    G = nx.from_numpy_matrix(adj_M)
+    G = nx.from_numpy_array(adj_M)
     color_ids = nx.coloring.greedy_color(
         G, strategy='independent_set', interchange=False
     )
@@ -182,5 +205,128 @@ def get_greedy_lut(lab, lut, ids=None):
 
     return greedy_lut
 
+def rgb_uint_to_html_hex(rgb):
+    r, g, b = rgb
+    hex_color = f'#{r:02x}{g:02x}{b:02x}'
+    return hex_color
 
-    
\ No newline at end of file
+def hex_to_rgb(hex):
+    if hex.startswith('#'):
+        hex = hex[1:]
+        
+    return tuple(int(hex[i:i+2], 16) for i in (0, 2, 4))
+
+def hierarchical_weights(alphas):
+    alphas = np.array([1.0, *alphas])
+    weights = []
+    for i, a_ref in enumerate(alphas):        
+        weight = np.prod(1-alphas[i+1:]) * a_ref
+        weights.append(weight)
+        
+    return weights[::-1]
+
+def hierarchical_blend(images, weights):
+    # Stack all images and do weighted sum
+    stacked = np.stack(images, axis=0)  # shape: (N, H, W)
+    return np.tensordot(weights, stacked, axes=(0, 0))
+
+def merge_two_grayscale_imgs(
+        img1, img2, rgb1, rgb2, alpha=0.5,
+        brightness1=1.0, brightness2=1.0, dtype=np.uint8, 
+        inverted=False
+    ):
+    if img1.max() > 1.0:
+        img1 = skimage.exposure.rescale_intensity(img1, out_range=(0, 1.0))
+        
+    if img2.max() > 1.0:
+        img2 = skimage.exposure.rescale_intensity(img2, out_range=(0, 1.0))
+        
+    img1_bright = np.clip(img1*brightness1, 0, 1.0) 
+    img2_bright = np.clip(img2*brightness1, 0, 1.0)     
+    
+    img1_rgb = (skimage.color.gray2rgb(img1_bright)*rgb1).astype(dtype)
+    img2_rgb = (skimage.color.gray2rgb(img2_bright)*rgb2).astype(dtype)
+    
+    merge = (alpha*img1_rgb + (1-alpha)*img2_rgb).astype(dtype)
+    
+    if inverted:
+        merge_inverted = merge.copy()
+        merge_inverted[..., 0] = 255-((merge[..., 1]+merge[..., 2])/2)
+        merge_inverted[..., 1] = 255-((merge[..., 0]+merge[..., 2])/2)
+        merge_inverted[..., 2] = 255-((merge[..., 1]+merge[..., 0])/2)
+        merge = merge_inverted
+
+    return merge
+
+def pg_ticks_to_colormap(ticks):
+    positions = []
+    colors = []
+    for pos, color in ticks:
+        positions.append(pos)
+        colors.append(color)
+    
+    cmap = ColorMap(positions, colors)
+    return cmap
+
+def color_palette(name='Okabe_lto', **sns_color_palette_kwargs):
+    """Create seaborn color palette (default or custom ones).
+
+    Parameters
+    ----------
+    name : str, optional
+        Name of the color palette. Default 'Okabe_lto'.
+    
+    References
+    ----------
+    https://thenode.biologists.com/data-visualization-with-flying-colors/research/
+    https://www.nature.com/articles/nmeth.1618
+    """
+    if name == 'Okabe_lto':
+        colors = (
+            '#0072B2', 
+            '#F0E442', 
+            '#009E73', 
+            '#56B4E9', 
+            '#E69F00', 
+            '#000000', 
+            '#CC79A7', 
+            '#D55E00', 
+        )
+        return sns.color_palette(colors)
+    elif name == 'Wong':
+        colors = (
+            (0, 0, 0), 
+            (230, 159, 0), 
+            (86, 180, 233), 
+            (0, 158, 115), 
+            (240, 228, 66), 
+            (0, 114, 178), 
+            (213, 94, 0), 
+            (204, 121, 167), 
+        )
+        return sns.color_palette(colors)
+    
+    return sns.color_palette(**sns_color_palette_kwargs)
+
+def grayscale_apply_lut(image, lut):
+    """
+    Map a grayscale image to RGBA using a lookup table.
+
+    Parameters
+    ----------
+    image : ndarray, shape (H, W)
+        Grayscale image with intensity values in [0, 1] or [0, 255].
+    lut : ndarray, shape (N, 4)
+        Lookup table of RGBA values (each channel in [0, 1] or [0, 255]).
+
+    Returns
+    -------
+    rgba : ndarray, shape (H, W, 4)
+        Colorized RGBA image.
+    """
+    # Normalize image to [0, N-1]
+    N = lut.shape[0]
+    img = np.clip(image, 0, 1) if image.dtype.kind == 'f' else image / 255.0
+    indices = np.clip((img * (N - 1)).astype(int), 0, N - 1)
+    rgba = lut[indices]
+    return rgba
\ No newline at end of file
