@@ -5,63 +5,20 @@ import traceback
 import difflib
 import numpy as np
 import pandas as pd
-import tkinter as tk
 import sys
-from tkinter import ttk
 
-from PyQt5.QtWidgets import (
-    QApplication, QPushButton, QHBoxLayout, QLabel, QSizePolicy
-)
-from PyQt5.QtCore import Qt
-from PyQt5.QtGui import QFont
+from . import GUI_INSTALLED
 
-from . import apps, myutils, printl, html_utils, load
+if GUI_INSTALLED:
+    from qtpy.QtWidgets import (
+        QApplication, QPushButton, QHBoxLayout, QLabel, QSizePolicy
+    )
+    from qtpy.QtCore import Qt
+    from qtpy.QtGui import QFont
+    from . import widgets, apps
 
-class RichTextPushButton(QPushButton):
-    def __init__(self, parent=None, text=None):
-        if parent is not None:
-            super().__init__(parent)
-        else:
-            super().__init__()
-        self.__lbl = QLabel(self)
-        if text is not None:
-            self.__lbl.setText(text)
-        self.__lyt = QHBoxLayout()
-        self.__lyt.setContentsMargins(5, 0, 0, 0)
-        self.__lyt.setSpacing(0)
-        self.setLayout(self.__lyt)
-        self.__lbl.setAttribute(Qt.WA_TranslucentBackground)
-        self.__lbl.setAttribute(Qt.WA_TransparentForMouseEvents)
-        self.__lbl.setSizePolicy(
-            QSizePolicy.Expanding,
-            QSizePolicy.Expanding,
-        )
-        self.__lbl.setTextFormat(Qt.RichText)
-        self.__lyt.addWidget(self.__lbl)
-        return
-
-    def setText(self, text):
-        self.__lbl.setText(text)
-        self.updateGeometry()
-        return
-
-    def sizeHint(self):
-        s = QPushButton.sizeHint(self)
-        w = self.__lbl.sizeHint()
-        s.setWidth(w.width()+10)
-        s.setHeight(w.height()+8)
-        return s
-
-def folder_dialog(toplevel=False, **options):
-    #Prompt the user to select the image file
-    if toplevel:
-        root = tk.Toplevel()
-    else:
-        root = tk.Tk()
-        root.withdraw()
-    path = tk.filedialog.Directory(**options).show()
-    root.destroy()
-    return path
+from . import myutils, printl, html_utils, load
+from . import settings_folderpath
 
 class select_channel_name:
     def __init__(self, which_channel=None, allow_abort=True):
@@ -73,7 +30,7 @@ class select_channel_name:
 
     def get_available_channels(
             self, filenames, images_path, useExt=None,
-            channelExt=('.h5', '.tif', '_aligned.npz'), 
+            channelExt=('.tif', '_aligned.npz'), 
             validEndnames=('aligned.npz', 'acdc_output.csv', 'segm.npz')
         ):
         # First check if metadata.csv already has the channel names
@@ -85,6 +42,7 @@ class select_channel_name:
         
         chNames_found = False
         channel_names = set()
+        basename = None
         if metadata_csv_path is not None:
             df = pd.read_csv(metadata_csv_path)
             basename = None
@@ -131,6 +89,7 @@ class select_channel_name:
                     ends = [
                         ext for ext in channelExt if (file.endswith(ext) 
                         and not file.endswith('btrack_tracks.h5'))
+                        and not file.endswith('edited.h5')
                     ]
                     if ends:
                         endName = file[len(basename):]
@@ -138,11 +97,13 @@ class select_channel_name:
                         if chName not in channel_names:
                             channel_names.append(chName)
                 return channel_names, False
-
+        
         # Find basename as intersection of filenames
         channel_names = set()
         self.basenameNotFound = False
         isBasenamePresent = myutils.checkDataIntegrity(filenames, images_path)
+        if basename is None:
+            basename = filenames[0]
         basename = filenames[0]
         for file in filenames:
             # Determine the basename based on intersection of all .tif
@@ -162,12 +123,20 @@ class select_channel_name:
             if not validFile:
                 continue
             sm = difflib.SequenceMatcher(None, file, basename)
-            i, j, k = sm.find_longest_match(0, len(file),
-                                            0, len(basename))
+            i, j, k = sm.find_longest_match(0, len(file), 0, len(basename))
+            if i > 0:
+                continue
             basename = file[i:i+k]
         self.basename = basename
+        
         basenameNotFound = [False]
         for file in filenames:
+            if file.endswith('edited.h5'):
+                continue
+            
+            if file.endswith('btrack_tracks.h5'):
+                continue
+            
             filename, ext = os.path.splitext(file)
             validImageFile = False
             if ext in channelExt:
@@ -175,10 +144,11 @@ class select_channel_name:
             elif file.endswith('aligned.npz'):
                 validImageFile = True
                 filename = filename[:-len('_aligned')]
+            
             if not validImageFile:
                 continue
-
-            channel_name = filename.split(basename)[-1]
+        
+            channel_name = filename.split(basename)[-1]            
             channel_names.add(channel_name)
             if channel_name == filename:
                 # Warn that an intersection could not be found
@@ -197,15 +167,14 @@ class select_channel_name:
                     idx = is_phase_contr_li.index(True)
                     channel_names[0], channel_names[idx] = (
                                       channel_names[idx], channel_names[0])
+        
         return channel_names, any(basenameNotFound)
 
     def _load_last_selection(self):
         last_sel_channel = None
         ch = self.which_channel
         if self.which_channel is not None:
-            _path = os.path.dirname(os.path.realpath(__file__))
-            temp_path = os.path.join(_path, 'temp')
-            txt_path = os.path.join(temp_path, f'{ch}_last_sel.txt')
+            txt_path = os.path.join(settings_folderpath, f'{ch}_last_sel.txt')
             if os.path.exists(txt_path):
                 with open(txt_path) as txt:
                     last_sel_channel = txt.read()
@@ -214,15 +183,14 @@ class select_channel_name:
     def _save_last_selection(self, selection):
         ch = self.which_channel
         if self.which_channel is not None:
-            _path = os.path.dirname(os.path.realpath(__file__))
-            temp_path = os.path.join(_path, 'temp')
-            if not os.path.exists(temp_path):
-                os.mkdir(temp_path)
-            txt_path = os.path.join(temp_path, f'{ch}_last_sel.txt')
+            if not os.path.exists(settings_folderpath):
+                os.mkdir(settings_folderpath)
+            txt_path = os.path.join(settings_folderpath, f'{ch}_last_sel.txt')
             with open(txt_path, 'w') as txt:
                 txt.write(selection)
     
     def askChannelName(self, filenames, images_path, ask, ch_names):
+        from . import apps
         if not ask:
             return ch_names
         filename = self.basename
@@ -258,12 +226,12 @@ class select_channel_name:
         df_metadata.at['channel_0_name', 'values'] = channel_name
         df_metadata.to_csv(metadata_csv_path)
         ch_names, _ = self.get_available_channels(filenames, images_path)
-        printl(ch_names)
         return ch_names
         
 
     def QtPrompt(self, parent, channel_names, informativeText='',
                  CbLabel='Select channel name:  '):
+        from . import apps
         font = QFont()
         font.setPixelSize(13)
         win = apps.QDialogCombobox(
@@ -293,74 +261,6 @@ class select_channel_name:
         else:
             self.user_ch_name = self.channel_name
 
-
-    def prompt(self, channel_names, message=None, toplevel=False):
-        if toplevel:
-            root = tk.Toplevel()
-        else:
-            root = tk.Tk()
-        root.lift()
-        # root.attributes("-topmost", True)
-        root.title('Select channel name')
-        root.geometry("+800+400")
-        row = 0
-        if message is not None:
-            tk.Label(root,
-                     text=message,
-                     font=(None, 11)).grid(row=row, column=0,
-                                           columnspan= 2, pady=(10,0),
-                                                          padx=10)
-            row += 1
-
-        tk.Label(root,
-                 text='Select channel name to analyse:'
-                 ).grid(row=row, column=0, pady=(10,0), padx=10)
-
-        ch_name_var = tk.StringVar()
-        w = max([len(s) for s in channel_names])+4
-        ch_name_combob = ttk.Combobox(root, width=w, justify='center',
-                                      textvariable=ch_name_var)
-        ch_name_combob.option_add('*TCombobox*Listbox.Justify', 'center')
-        ch_name_combob['values'] = channel_names
-        ch_name_combob.grid(column=1, row=row, padx=10, pady=(10,0))
-        if self.last_sel_channel is not None:
-            if self.last_sel_channel in channel_names:
-                ch_name_combob.current(channel_names.index(self.last_sel_channel))
-            else:
-                ch_name_combob.current(0)
-        else:
-            ch_name_combob.current(0)
-
-        ch_name_var.trace_add("write", self._test)
-
-        row += 1
-        tk.Button(root, text='Ok', width=20,
-                        command=self._tk_close).grid(row=row, column=0,
-                                                  columnspan=2,
-                                                  pady=10, padx=10)
-
-
-
-        root.protocol("WM_DELETE_WINDOW", self._tk_abort)
-        self.ch_name_var = ch_name_var
-        self.root = root
-
-        root.mainloop()
-
-    def _tk_close(self):
-        self.was_aborted = False
-        self.channel_name = self.ch_name_var.get()
-        self.root.quit()
-        self.root.destroy()
-
-
-    def _tk_abort(self):
-        self.was_aborted = True
-        self.root.quit()
-        self.root.destroy()
-        if self.allow_abort:
-            exit('Execution aborted by the user')
-
     def _test(self, name=None, index=None, mode=None):
         pass
 
@@ -369,14 +269,214 @@ class select_channel_name:
         if self.allow_abort:
             exit('Execution aborted by the user')
 
-def askyesno(title='tk', message='Yes or no?', toplevel=False):
-    if toplevel:
-        root = tk.Toplevel()
-    else:
-        root = tk.Tk()
-        root.withdraw()
-    yes = tk.messagebox.askyesno(title, message, master=root)
-    if not toplevel:
-        root.quit()
-        root.destroy()
-    return yes
+def exportToImageFinished(filepath, qparent=None):
+    from cellacdc import widgets
+    
+    txt = 'Exporting to image done!'
+    txt = f'{txt}<br><br>Files were saved here:'
+    
+    txt = html_utils.paragraph(txt)
+    msg = widgets.myMessageBox(wrapText=False)
+    msg.information(
+        qparent, 'Exporting image finished', txt, 
+        commands=(filepath,), 
+        path_to_browse=os.path.dirname(filepath)
+    )
+
+def exportToVideoFinished(
+        preferences, conversion_to_mp4_successful, qparent=None
+    ):
+    from cellacdc import widgets
+    
+    txt = 'Exporting to video finished!'
+    
+    msg_type = 'information'
+    if not conversion_to_mp4_successful:
+        from . import urls
+        github_href = html_utils.href_tag('GitHub page', urls.issues_url)
+        msg_type = 'warning'
+        txt = (
+            f'{txt}<br><br>'
+            'WARNING: <b>Conversion to MP4 failed</b>. '
+            'Video file was saved as AVI instead. '
+            f'Feel free to report the issue on our {github_href}'
+        )
+    
+    txt = f'{txt}<br><br>Files were saved here:'
+    
+    txt = html_utils.paragraph(txt)
+    
+    
+    folderpath = os.path.dirname(preferences['filepath'])
+    commands = [preferences['filepath']]
+    if preferences['save_pngs']:
+        commands.append(preferences['pngs_folderpath'])
+    
+    msg = widgets.myMessageBox(wrapText=False)
+    getattr(msg, msg_type)(
+        qparent, 'Exporting video finished', txt, 
+        commands=commands, path_to_browse=folderpath
+    )
+
+def askSamSaveEmbeddings(qparent=None):
+    txt = html_utils.paragraph("""
+    Segment Anything Model generates image embeddings that you 
+    can use later in module 3<br>
+    for <b>much faster interactive segmentation</b> (with points or bounding boxes 
+    prompts).<br><br>
+    Do you want to <b>save the image embeddings</b>?
+    """)
+    saveOnlyButton = widgets.BedPushButton('Save only embeddings')
+    saveButton = widgets.BedPlusLabelPushButton('Save also embeddings')
+    saveOnlyButton = widgets.BedPushButton('Save only embeddings')
+    msg = widgets.myMessageBox(wrapText=False)
+    _, saveOnlyButton, saveButton, _ = msg.question(
+        qparent, 'Save SAM Image Embeddings?', txt, 
+        buttonsTexts=(
+            'Cancel', saveOnlyButton, saveButton, 
+            widgets.NoBedPushButton('Do not save embeddings')
+        )
+    )
+    sam_only_embeddings = msg.clickedButton == saveOnlyButton
+    sam_also_embeddings = msg.clickedButton == saveButton
+    return sam_only_embeddings, sam_also_embeddings, msg.cancel
+
+def askSamLoadEmbeddings(
+        sam_embeddings_path, qparent=None, is_gui_caller=False,
+    ):
+    txt = html_utils.paragraph("""
+    Cell-ACDC detected <b>previously saved Segment Anything image embeddings</b> 
+    (see file path below).<br><br>
+    If you load the embeddings, computation time will be much lower.<br><br>
+    Do you want to <b>load the image embeddings</b>?
+    """)
+    msg = widgets.myMessageBox(wrapText=False)
+    loadButton = widgets.BedPlusLabelPushButton('Load embeddings and segment')
+    doNotLoadButton = widgets.NoBedPushButton('Do not load embeddings')
+    buttons = (loadButton, doNotLoadButton)
+    if is_gui_caller:
+        loadOnlyEmbedButton = widgets.BedPushButton('Only load embeddings')
+        buttons = (loadOnlyEmbedButton, *buttons)
+    
+    msg.question(
+        qparent, 'Load SAM Image Embeddings?', txt, 
+        buttonsTexts=('Cancel', *buttons), 
+        commands=(sam_embeddings_path,), 
+        path_to_browse=os.path.dirname(sam_embeddings_path)
+    )
+    loadEmbed = msg.clickedButton == loadButton
+    onlyLoadEmbed = False
+    if is_gui_caller:
+        onlyLoadEmbed = msg.clickedButton == loadOnlyEmbedButton
+    return loadEmbed, onlyLoadEmbed, msg.cancel
+
+def init_prompt_model_params(
+        posData, model_name, init_params, segment_params, 
+        qparent=None, help_url=None, init_last_params=False, 
+        ini_filename=None
+    ):
+    out = {}
+    
+    segm_files = load.get_segm_files(posData.images_path)
+    existingSegmEndnames = load.get_endnames(
+        posData.basename, segm_files
+    )
+    win = apps.QDialogModelParams(
+        init_params,
+        segment_params,
+        model_name, 
+        parent=qparent,
+        url=help_url, 
+        initLastParams=init_last_params, 
+        posData=posData,
+        segmFileEndnames=existingSegmEndnames,
+        df_metadata=posData.metadata_df,
+        addPreProcessParams=False,
+        addPostProcessParams=False,
+        ini_filename=ini_filename,
+        add_additional_segm_params=False
+    )
+    win.setChannelNames(posData.chNames)
+    out['win'] = win
+    win.exec_()
+    return out
+
+def init_segm_model_params(
+        posData, model_name, init_params, segment_params, 
+        qparent=None, help_url=None, init_last_params=False, 
+        check_sam_embeddings=True, is_gui_caller=False,
+        extraParams=None, extraParamsTitle=None,
+        ini_filename=None, add_additional_segm_params=False
+    ):
+    out = {}
+    
+    is_sam_model = (
+        model_name == 'segment_anything' and check_sam_embeddings
+    )
+    
+    # If SAM with prompts and embeddings were prev saved, asks to load them
+    load_sam_embed = False
+    only_load_sam_embed = False
+    sam_embeddings_exist = os.path.exists(posData.sam_embeddings_path)
+    sam_embeddings_loaded = hasattr(posData, 'sam_embeddings')
+    if is_sam_model and sam_embeddings_exist and not sam_embeddings_loaded:
+        load_sam_embed, only_load_sam_embed, cancel = askSamLoadEmbeddings(
+            posData.sam_embeddings_path, qparent=qparent, 
+            is_gui_caller=is_gui_caller
+        )
+        if cancel:
+            return out
+    
+    out['load_sam_embeddings'] = only_load_sam_embed or load_sam_embed
+    if only_load_sam_embed:
+        return out
+    
+    segm_files = load.get_segm_files(posData.images_path)
+    existingSegmEndnames = load.get_endnames(
+        posData.basename, segm_files
+    )
+    win = apps.QDialogModelParams(
+        init_params,
+        segment_params,
+        model_name, parent=qparent,
+        url=help_url, 
+        initLastParams=init_last_params, 
+        posData=posData,
+        segmFileEndnames=existingSegmEndnames,
+        df_metadata=posData.metadata_df,
+        force_postprocess_2D=False,
+        extraParams=extraParams,
+        extraParamsTitle=extraParamsTitle,
+        ini_filename=ini_filename,
+        add_additional_segm_params=add_additional_segm_params
+    )
+    win.setChannelNames(posData.chNames)
+    out['win'] = win
+    win.exec_()
+    if win.cancel:
+        return out
+    
+    if load_sam_embed:
+        win.model_kwargs['use_loaded_embeddings'] = True
+        posData.loadSamEmbeddings()
+    
+    ask_sam_embeddings = (
+        model_name == 'segment_anything' 
+        and not load_sam_embed
+        and check_sam_embeddings
+    )
+    # If SAM and embeddings were not laoded, asks to save them
+    if ask_sam_embeddings:
+        sam_only_embeddings, sam_also_embeddings, cancel = (
+            askSamSaveEmbeddings(qparent=qparent)
+        )
+        if cancel:
+            return out
+
+        win.model_kwargs['only_embeddings'] = sam_only_embeddings
+        win.model_kwargs['save_embeddings'] = (
+            sam_only_embeddings or sam_also_embeddings
+        )
+    
+    return out
+    
\ No newline at end of file
